@@ -6,7 +6,7 @@
 /* SYNOPSIS
 /*	\fBqmgr\fR [generic Postfix daemon options]
 /* DESCRIPTION
-/*	The \fBqmgr\fR daemon awaits the arrival of incoming mail
+/*	The \fBqmgr\fR(8) daemon awaits the arrival of incoming mail
 /*	and arranges for its delivery via Postfix delivery processes.
 /*	The actual mail routing strategy is delegated to the
 /*	\fBtrivial-rewrite\fR(8) daemon.
@@ -19,10 +19,10 @@
 /* MAIL QUEUES
 /* .ad
 /* .fi
-/*	The \fBqmgr\fR daemon maintains the following queues:
+/*	The \fBqmgr\fR(8) daemon maintains the following queues:
 /* .IP \fBincoming\fR
 /*	Inbound mail from the network, or mail picked up by the
-/*	local \fBpickup\fR agent from the \fBmaildrop\fR directory.
+/*	local \fBpickup\fR(8) agent from the \fBmaildrop\fR directory.
 /* .IP \fBactive\fR
 /*	Messages that the queue manager has opened for delivery. Only
 /*	a limited number of messages is allowed to enter the \fBactive\fR
@@ -39,7 +39,7 @@
 /* DELIVERY STATUS REPORTS
 /* .ad
 /* .fi
-/*	The \fBqmgr\fR daemon keeps an eye on per-message delivery status
+/*	The \fBqmgr\fR(8) daemon keeps an eye on per-message delivery status
 /*	reports in the following directories. Each status report file has
 /*	the same name as the corresponding message file:
 /* .IP \fBbounce\fR
@@ -53,7 +53,7 @@
 /*	Postfix "\fBsendmail -v\fR" or "\fBsendmail -bv\fR" command.
 /*	These files are maintained by the \fBtrace\fR(8) daemon.
 /* .PP
-/*	The \fBqmgr\fR daemon is responsible for asking the
+/*	The \fBqmgr\fR(8) daemon is responsible for asking the
 /*	\fBbounce\fR(8), \fBdefer\fR(8) or \fBtrace\fR(8) daemons to
 /*	send delivery reports.
 /* STRATEGIES
@@ -109,26 +109,25 @@
 /*	servers that should not go away forever. The action is to start
 /*	an incoming queue scan.
 /* .PP
-/*	The \fBqmgr\fR daemon reads an entire buffer worth of triggers.
+/*	The \fBqmgr\fR(8) daemon reads an entire buffer worth of triggers.
 /*	Multiple identical trigger requests are collapsed into one, and
 /*	trigger requests are sorted so that \fBA\fR and \fBF\fR precede
 /*	\fBD\fR and \fBI\fR. Thus, in order to force a deferred queue run,
 /*	one would request \fBA F D\fR; in order to notify the queue manager
 /*	of the arrival of new mail one would request \fBI\fR.
 /* STANDARDS
-/* .ad
-/* .fi
-/*	None. The \fBqmgr\fR daemon does not interact with the outside world.
+/*	RFC 3463 (Enhanced status codes)
+/*	RFC 3464 (Delivery status notifications)
 /* SECURITY
 /* .ad
 /* .fi
-/*	The \fBqmgr\fR daemon is not security sensitive. It reads
+/*	The \fBqmgr\fR(8) daemon is not security sensitive. It reads
 /*	single-character messages from untrusted local users, and thus may
-/*	be susceptible to denial of service attacks. The \fBqmgr\fR daemon
+/*	be susceptible to denial of service attacks. The \fBqmgr\fR(8) daemon
 /*	does not talk to the outside world, and it can be run at fixed low
 /*	privilege in a chrooted environment.
 /* DIAGNOSTICS
-/*	Problems and transactions are logged to the syslog daemon.
+/*	Problems and transactions are logged to the \fBsyslog\fR(8) daemon.
 /*	Corrupted message files are saved to the \fBcorrupt\fR queue
 /*	for further inspection.
 /*
@@ -136,17 +135,18 @@
 /*	the postmaster is notified of bounces and of other trouble.
 /* BUGS
 /*	A single queue manager process has to compete for disk access with
-/*	multiple front-end processes such as \fBsmtpd\fR. A sudden burst of
+/*	multiple front-end processes such as \fBcleanup\fR(8). A sudden burst of
 /*	inbound mail can negatively impact outbound delivery rates.
 /* CONFIGURATION PARAMETERS
 /* .ad
 /* .fi
-/*	Changes to \fBmain.cf\fR are not picked up automatically, as qmgr(8)
+/*	Changes to \fBmain.cf\fR are not picked up automatically,
+/*	as \fBqmgr\fR(8)
 /*	is a persistent process. Use the command "\fBpostfix reload\fR" after
 /*	a configuration change.
 /*
 /*	The text below provides only a parameter summary. See
-/*	postconf(5) for more details including examples.
+/*	\fBpostconf\fR(5) for more details including examples.
 /*
 /*	In the text below, \fItransport\fR is the first field in a
 /*	\fBmaster.cf\fR entry.
@@ -219,8 +219,11 @@
 /*	How much time a Postfix daemon process may take to handle a
 /*	request before it is terminated by a built-in watchdog timer.
 /* .IP "\fBdefer_transports (empty)\fR"
-/*	The names of message delivery transports that should not be delivered
-/*	to unless someone issues "\fBsendmail -q\fR" or equivalent.
+/*	The names of message delivery transports that should not deliver mail
+/*	unless someone issues "\fBsendmail -q\fR" or equivalent.
+/* .IP "\fBdelay_logging_resolution_limit (2)\fR"
+/*	The maximal number of digits after the decimal point when logging
+/*	sub-second delay values.
 /* .IP "\fBhelpful_warnings (yes)\fR"
 /*	Log warnings about problematic configuration settings, and provide
 /*	helpful suggestions.
@@ -251,7 +254,7 @@
 /*	postconf(5), configuration parameters
 /*	master(5), generic daemon options
 /*	master(8), process manager
-/*	syslogd(8) system logging
+/*	syslogd(8), system logging
 /* README FILES
 /* .ad
 /* .fi
@@ -325,7 +328,6 @@ int     var_local_rcpt_lim;		/* XXX */
 int     var_local_con_lim;		/* XXX */
 int     var_proc_limit;
 bool    var_verp_bounce_off;
-bool    var_sender_routing;
 int     var_qmgr_clog_warn_time;
 
 static QMGR_SCAN *qmgr_incoming;
@@ -492,10 +494,15 @@ static void qmgr_post_init(char *unused_name, char **unused_argv)
      * Sanity check.
      */
     if (var_qmgr_rcpt_limit < var_qmgr_active_limit) {
-	msg_warn("%s is smaller than %s",
-		 VAR_QMGR_RCPT_LIMIT, VAR_QMGR_ACT_LIMIT);
+	msg_warn("%s is smaller than %s - adjusting %s",
+	      VAR_QMGR_RCPT_LIMIT, VAR_QMGR_ACT_LIMIT, VAR_QMGR_RCPT_LIMIT);
 	var_qmgr_rcpt_limit = var_qmgr_active_limit;
     }
+    if (var_dsn_queue_time > var_max_queue_time) {
+	msg_warn("%s is larger than %s - adjusting %s",
+		 VAR_DSN_QUEUE_TIME, VAR_MAX_QUEUE_TIME, VAR_DSN_QUEUE_TIME);
+	var_dsn_queue_time = var_max_queue_time;
+    }
 
     /*
      * This routine runs after the skeleton code has entered the chroot jail.
@@ -551,7 +558,6 @@ int     main(int argc, char **argv)
     static CONFIG_BOOL_TABLE bool_table[] = {
 	VAR_ALLOW_MIN_USER, DEF_ALLOW_MIN_USER, &var_allow_min_user,
 	VAR_VERP_BOUNCE_OFF, DEF_VERP_BOUNCE_OFF, &var_verp_bounce_off,
-	VAR_SENDER_ROUTING, DEF_SENDER_ROUTING, &var_sender_routing,
 	0,
     };
 
