@@ -6,7 +6,7 @@
 /* SYNOPSIS
 /*	\fBpostcat\fR [\fB-oqv\fR] [\fB-c \fIconfig_dir\fR] [\fIfiles\fR...]
 /* DESCRIPTION
-/*	The \fBpostcat\fR command prints the contents of the named
+/*	The \fBpostcat\fR(1) command prints the contents of the named
 /*	\fIfiles\fR in human-readable form. The files are expected
 /*	to be in Postfix queue file format. If no
 /*	\fIfiles\fR are specified on the command line, the program
@@ -40,7 +40,7 @@
 /*	this program.
 /*
 /*	The text below provides only a parameter summary. See
-/*	postconf(5) for more details including examples.
+/*	\fBpostconf\fR(5) for more details including examples.
 /* .IP "\fBconfig_directory (see 'postconf -d' output)\fR"
 /*	The default location of the Postfix main.cf and master.cf
 /*	configuration files.
@@ -65,6 +65,7 @@
 
 #include <sys_defs.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
@@ -78,6 +79,7 @@
 #include <vstring.h>
 #include <msg_vstream.h>
 #include <vstring_vstream.h>
+#include <stringops.h>
 
 /* Global library. */
 
@@ -86,6 +88,7 @@
 #include <mail_queue.h>
 #include <mail_conf.h>
 #include <mail_params.h>
+#include <mail_proto.h>
 
 /* Application-specific. */
 
@@ -101,11 +104,16 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 {
     int     prev_type = 0;
     int     rec_type;
+    struct timeval tv;
     time_t  time;
     int     first = 1;
     int     ch;
     off_t   offset;
     int     in_message = 0;
+    const char *error_text;
+    char   *attr_name;
+    char   *attr_value;
+    int     rec_flags = (msg_verbose ? REC_FLAG_NONE : REC_FLAG_DEFAULT);
 
 #define TEXT_RECORD(rec_type) \
 	    (rec_type == REC_TYPE_CONT || rec_type == REC_TYPE_NORM)
@@ -124,10 +132,10 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
     /*
      * Now look at the rest.
      */
-    for (;;) {
+    do {
 	if (flags & PC_FLAG_OFFSET)
 	    offset = vstream_ftell(fp);
-	rec_type = rec_get(fp, buffer, 0);
+	rec_type = rec_get_raw(fp, buffer, 0, rec_flags);
 	if (rec_type == REC_TYPE_ERROR)
 	    msg_fatal("record read error");
 	if (rec_type == REC_TYPE_EOF)
@@ -142,13 +150,25 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	    vstream_printf("%9lu ", (unsigned long) offset);
 	switch (rec_type) {
 	case REC_TYPE_TIME:
-	case REC_TYPE_WARN:
-	    time = atol(STR(buffer));
+	    REC_TYPE_TIME_SCAN(STR(buffer), tv);
+	    time = tv.tv_sec;
 	    vstream_printf("%s: %s", rec_type_name(rec_type),
 			   asctime(localtime(&time)));
 	    break;
+	case REC_TYPE_WARN:
+	    REC_TYPE_WARN_SCAN(STR(buffer), time);
+	    vstream_printf("%s: %s", rec_type_name(rec_type),
+			   asctime(localtime(&time)));
+	    break;
+	case REC_TYPE_PTR:			/* pointer */
+	    vstream_printf("%s: ", rec_type_name(rec_type));
+	    vstream_fwrite(VSTREAM_OUT, STR(buffer), LEN(buffer));
+	    VSTREAM_PUTCHAR('\n');
+	    if (rec_goto(fp, STR(buffer)) == REC_TYPE_ERROR)
+		msg_fatal("bad pointer record, or input is not seekable");
+	    break;
 	case REC_TYPE_CONT:			/* REC_TYPE_FILT collision */
-	    if (!in_message) 
+	    if (!in_message)
 		vstream_printf("%s: ", rec_type_name(rec_type));
 	    else if (msg_verbose)
 		vstream_printf("unterminated_text: ");
@@ -164,6 +184,13 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	    vstream_fwrite(VSTREAM_OUT, STR(buffer), LEN(buffer));
 	    VSTREAM_PUTCHAR('\n');
 	    break;
+	case REC_TYPE_DTXT:
+	    if (msg_verbose) {
+		vstream_printf("%s: ", rec_type_name(rec_type));
+		vstream_fwrite(VSTREAM_OUT, STR(buffer), LEN(buffer));
+		VSTREAM_PUTCHAR('\n');
+	    }
+	    break;
 	case REC_TYPE_MESG:
 	    vstream_printf("*** MESSAGE CONTENTS %s ***\n", VSTREAM_PATH(fp));
 	    in_message = 1;
@@ -175,6 +202,22 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	case REC_TYPE_END:
 	    vstream_printf("*** MESSAGE FILE END %s ***\n", VSTREAM_PATH(fp));
 	    break;
+	case REC_TYPE_ATTR:
+	    error_text = split_nameval(STR(buffer), &attr_name, &attr_value);
+	    if (error_text != 0) {
+		msg_warn("%s: malformed attribute: %s: %.100s",
+			 VSTREAM_PATH(fp), error_text, STR(buffer));
+		break;
+	    }
+	    if (strcmp(attr_name, MAIL_ATTR_CREATE_TIME) == 0) {
+		time = atol(attr_value);
+		vstream_printf("%s: %s", MAIL_ATTR_CREATE_TIME,
+			       asctime(localtime(&time)));
+		break;
+	    }
+	    vstream_printf("%s: %s=%s\n", rec_type_name(rec_type),
+			   attr_name, attr_value);
+	    break;
 	default:
 	    vstream_printf("%s: %s\n", rec_type_name(rec_type), STR(buffer));
 	    break;
@@ -185,7 +228,7 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	 * In case the next record is broken.
 	 */
 	vstream_fflush(VSTREAM_OUT);
-    }
+    } while (rec_type != REC_TYPE_END);
 }
 
 /* usage - explain and terminate */
