@@ -17,7 +17,7 @@
 #include <name_code.h>
 
  /*
-  * TLS enforcement levels. Non-sentinel values also be used to indicate
+  * TLS enforcement levels. Non-sentinel values may also be used to indicate
   * the actual security level of a session.
   */
 #define TLS_LEV_NOTFOUND	-1	/* sentinel */
@@ -174,6 +174,7 @@ typedef struct {
     int     verifydepth;
     const char *cache_type;
     long    scache_timeout;
+    int     set_sessid;
     const char *cert_file;
     const char *key_file;
     const char *dcert_file;
